@@ -38,6 +38,12 @@
 #include "gstjpegenc.h"
 #include <gst/video/video.h>
 
+/* experimental */
+/* setting smoothig seems to have no effect in libjepeg
+#define ENABLE_SMOOTHING 1
+*/
+/*#define ENABLE_COLORSPACE_RGB 1 */
+
 /* elementfactory information */
 static const GstElementDetails gst_jpegenc_details =
 GST_ELEMENT_DETAILS ("JPEG image encoder",
@@ -49,6 +55,8 @@ GST_DEBUG_CATEGORY_STATIC (jpegenc_debug);
 #define GST_CAT_DEFAULT jpegenc_debug
 
 #define JPEG_DEFAULT_QUALITY 85
+#define JPEG_DEFAULT_SMOOTHING 0
+#define JPEG_DEFAULT_IDCT_METHOD	JDCT_FASTEST
 
 /* These macros are adapted from videotestsrc.c 
  *  and/or gst-plugins/gst/games/gstvideoimage.c */
@@ -74,12 +82,15 @@ enum
 
 enum
 {
-  ARG_0,
-  ARG_QUALITY,
-  ARG_SMOOTHING
-      /* FILL ME */
+  PROP_0,
+  PROP_QUALITY,
+  PROP_SMOOTHING,
+  PROP_IDCT_METHOD
 };
 
+extern GType gst_idct_method_get_type (void);
+#define GST_TYPE_IDCT_METHOD (gst_idct_method_get_type())
+
 static void gst_jpegenc_base_init (gpointer g_class);
 static void gst_jpegenc_class_init (GstJpegEnc * klass);
 static void gst_jpegenc_init (GstJpegEnc * jpegenc);
@@ -174,16 +185,22 @@ gst_jpegenc_class_init (GstJpegEnc * klass)
   gobject_class->get_property = gst_jpegenc_get_property;
 
 
-  g_object_class_install_property (gobject_class, ARG_QUALITY,
+  g_object_class_install_property (gobject_class, PROP_QUALITY,
       g_param_spec_int ("quality", "Quality", "Quality of encoding",
           0, 100, JPEG_DEFAULT_QUALITY, G_PARAM_READWRITE));
-#if 0
+
+#if ENABLE_SMOOTHING
   /* disabled, since it doesn't seem to work */
-  g_object_class_install_property (gobject_class, ARG_SMOOTHING,
+  g_object_class_install_property (gobject_class, PROP_SMOOTHING,
       g_param_spec_int ("smoothing", "Smoothing", "Smoothing factor",
-          0, 100, 0, G_PARAM_READWRITE));
+          0, 100, JPEG_DEFAULT_SMOOTHING, G_PARAM_READWRITE));
 #endif
 
+  g_object_class_install_property (gobject_class, PROP_IDCT_METHOD,
+      g_param_spec_enum ("idct-method", "IDCT Method",
+          "The IDCT algorithm to use", GST_TYPE_IDCT_METHOD,
+          JPEG_DEFAULT_IDCT_METHOD, G_PARAM_READWRITE));
+
   gstelement_class->change_state = gst_jpegenc_change_state;
 
   gobject_class->finalize = gst_jpegenc_finalize;
@@ -201,14 +218,52 @@ gst_jpegenc_init_destination (j_compress_ptr cinfo)
 static boolean
 gst_jpegenc_flush_destination (j_compress_ptr cinfo)
 {
-  GST_DEBUG ("gst_jpegenc_chain: flush_destination: buffer too small !!!");
+  GstBuffer *overflow_buffer;
+  guint32 old_buffer_size;
+  GstJpegEnc *jpegenc = (GstJpegEnc *) (cinfo->client_data);
+  GST_DEBUG_OBJECT (jpegenc,
+      "gst_jpegenc_chain: flush_destination: buffer too small");
+
+  /* Our output buffer wasn't big enough.
+   * Make a new buffer that's twice the size, */
+  old_buffer_size = GST_BUFFER_SIZE (jpegenc->output_buffer);
+  gst_pad_alloc_buffer_and_set_caps (jpegenc->srcpad,
+      GST_BUFFER_OFFSET_NONE, old_buffer_size * 2,
+      GST_PAD_CAPS (jpegenc->srcpad), &overflow_buffer);
+  memcpy (GST_BUFFER_DATA (overflow_buffer),
+      GST_BUFFER_DATA (jpegenc->output_buffer), old_buffer_size);
+
+  gst_buffer_copy_metadata (overflow_buffer, jpegenc->output_buffer,
+      GST_BUFFER_COPY_TIMESTAMPS);
+
+  /* drop it into place, */
+  gst_buffer_unref (jpegenc->output_buffer);
+  jpegenc->output_buffer = overflow_buffer;
+
+  /* and last, update libjpeg on where to work. */
+  jpegenc->jdest.next_output_byte =
+      GST_BUFFER_DATA (jpegenc->output_buffer) + old_buffer_size;
+  jpegenc->jdest.free_in_buffer =
+      GST_BUFFER_SIZE (jpegenc->output_buffer) - old_buffer_size;
+
   return TRUE;
 }
 
 static void
 gst_jpegenc_term_destination (j_compress_ptr cinfo)
 {
-  GST_DEBUG ("gst_jpegenc_chain: term_source");
+  GstJpegEnc *jpegenc = (GstJpegEnc *) (cinfo->client_data);
+  GST_DEBUG_OBJECT (jpegenc, "gst_jpegenc_chain: term_source");
+
+  /* Trim the buffer size and push it. */
+  GST_BUFFER_SIZE (jpegenc->output_buffer) =
+      GST_ROUND_UP_4 (GST_BUFFER_SIZE (jpegenc->output_buffer) -
+      jpegenc->jdest.free_in_buffer);
+
+  g_signal_emit (G_OBJECT (jpegenc), gst_jpegenc_signals[FRAME_ENCODED], 0);
+
+  jpegenc->last_ret = gst_pad_push (jpegenc->srcpad, jpegenc->output_buffer);
+  jpegenc->output_buffer = NULL;
 }
 
 static void
@@ -247,15 +302,18 @@ gst_jpegenc_init (GstJpegEnc * jpegenc)
   jpegenc->jdest.empty_output_buffer = gst_jpegenc_flush_destination;
   jpegenc->jdest.term_destination = gst_jpegenc_term_destination;
   jpegenc->cinfo.dest = &jpegenc->jdest;
+  jpegenc->cinfo.client_data = jpegenc;
 
+
+  /* init properties */
   jpegenc->quality = JPEG_DEFAULT_QUALITY;
-  jpegenc->smoothing = 0;
+  jpegenc->smoothing = JPEG_DEFAULT_SMOOTHING;
+  jpegenc->idct_method = JPEG_DEFAULT_IDCT_METHOD;
 }
 
 static void
 gst_jpegenc_finalize (GObject * object)
 {
-
   GstJpegEnc *filter = GST_JPEGENC (object);
 
   jpeg_destroy_compress (&filter->cinfo);
@@ -360,8 +418,7 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
 
   GST_DEBUG_OBJECT (jpegenc, "width %d, height %d", width, height);
 
-
-#if 0
+#if ENABLE_COLORSPACE_RGB
   switch (jpegenc->format) {
     case GST_COLORSPACE_RGB24:
       jpegenc->bufsize = jpegenc->width * jpegenc->height * 3;
@@ -377,12 +434,13 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
       jpegenc->cinfo.in_color_space = JCS_YCbCr;
 
       jpeg_set_defaults (&jpegenc->cinfo);
-      jpeg_set_quality (&jpegenc->cinfo, jpegenc->quality, TRUE);
+      /* these are set in _chain()
+         jpeg_set_quality (&jpegenc->cinfo, jpegenc->quality, TRUE);
+         jpegenc->cinfo.smoothing_factor = jpegenc->smoothing;
+         jpegenc->cinfo.dct_method = jpegenc->idct_method;
+       */
 
       jpegenc->cinfo.raw_data_in = TRUE;
-      jpegenc->cinfo.dct_method = JDCT_FASTEST;
-      /*jpegenc->cinfo.dct_method = JDCT_DEFAULT; */
-      /*jpegenc->cinfo.smoothing_factor = jpegenc->smoothing; */
 
       if (height != -1) {
         jpegenc->line[0] =
@@ -394,7 +452,7 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
       }
 
       GST_DEBUG_OBJECT (jpegenc, "setting format done");
-#if 0
+#if ENABLE_COLORSPACE_RGB
       break;
     default:
       printf ("gst_jpegenc_resync: unsupported colorspace, using RGB\n");
@@ -404,8 +462,11 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
   }
 #endif
 
+  /* guard against a potential error in gst_jpegenc_term_destination
+     which occurs iff bufsize % 4 < free_space_remaining */
+  jpegenc->bufsize = GST_ROUND_UP_4 (jpegenc->bufsize);
+
   jpeg_suppress_tables (&jpegenc->cinfo, TRUE);
-  //jpeg_suppress_tables(&jpegenc->cinfo, FALSE);
 
   GST_DEBUG_OBJECT (jpegenc, "resync done");
 }
@@ -417,7 +478,6 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   GstJpegEnc *jpegenc;
   guchar *data;
   gulong size;
-  GstBuffer *outbuf;
   guint height, width;
   guchar *base[3], *end[3];
   gint i, j, k;
@@ -430,17 +490,18 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
 
-  GST_DEBUG_OBJECT (jpegenc, "got buffer of %lu bytes", size);
+  GST_LOG_OBJECT (jpegenc, "got buffer of %lu bytes", size);
 
   ret =
       gst_pad_alloc_buffer_and_set_caps (jpegenc->srcpad,
       GST_BUFFER_OFFSET_NONE, jpegenc->bufsize, GST_PAD_CAPS (jpegenc->srcpad),
-      &outbuf);
+      &jpegenc->output_buffer);
 
   if (ret != GST_FLOW_OK)
     goto done;
 
-  gst_buffer_copy_metadata (outbuf, buf, GST_BUFFER_COPY_TIMESTAMPS);
+  gst_buffer_copy_metadata (jpegenc->output_buffer, buf,
+      GST_BUFFER_COPY_TIMESTAMPS);
 
   width = jpegenc->width;
   height = jpegenc->height;
@@ -453,14 +514,19 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   end[1] = base[1] + (height / 2) * I420_U_ROWSTRIDE (width);
   end[2] = base[2] + (height / 2) * I420_V_ROWSTRIDE (width);
 
-  jpegenc->jdest.next_output_byte = GST_BUFFER_DATA (outbuf);
-  jpegenc->jdest.free_in_buffer = GST_BUFFER_SIZE (outbuf);
+  jpegenc->jdest.next_output_byte = GST_BUFFER_DATA (jpegenc->output_buffer);
+  jpegenc->jdest.free_in_buffer = GST_BUFFER_SIZE (jpegenc->output_buffer);
 
+  /* prepare for raw input */
+#if JPEG_LIB_VERSION >= 70
+  jpegenc->cinfo.do_fancy_downsampling = FALSE;
+#endif
   jpegenc->cinfo.smoothing_factor = jpegenc->smoothing;
+  jpegenc->cinfo.dct_method = jpegenc->idct_method;
   jpeg_set_quality (&jpegenc->cinfo, jpegenc->quality, TRUE);
   jpeg_start_compress (&jpegenc->cinfo, TRUE);
 
-  GST_DEBUG_OBJECT (jpegenc, "compressing");
+  GST_LOG_OBJECT (jpegenc, "compressing");
 
   for (i = 0; i < height; i += 2 * DCTSIZE) {
     /*g_print ("next scanline: %d\n", jpegenc->cinfo.next_scanline); */
@@ -481,15 +547,10 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
     jpeg_write_raw_data (&jpegenc->cinfo, jpegenc->line, 2 * DCTSIZE);
   }
 
+  /* This will ensure that gst_jpegenc_term_destination is called; we push
+     the final output buffer from there */
   jpeg_finish_compress (&jpegenc->cinfo);
-  GST_DEBUG_OBJECT (jpegenc, "compressing done");
-
-  GST_BUFFER_SIZE (outbuf) =
-      GST_ROUND_UP_4 (jpegenc->bufsize - jpegenc->jdest.free_in_buffer);
-
-  g_signal_emit (G_OBJECT (jpegenc), gst_jpegenc_signals[FRAME_ENCODED], 0);
-
-  ret = gst_pad_push (jpegenc->srcpad, outbuf);
+  GST_LOG_OBJECT (jpegenc, "compressing done");
 
 done:
   gst_buffer_unref (buf);
@@ -514,12 +575,17 @@ gst_jpegenc_set_property (GObject * object, guint prop_id,
   GST_OBJECT_LOCK (jpegenc);
 
   switch (prop_id) {
-    case ARG_QUALITY:
+    case PROP_QUALITY:
       jpegenc->quality = g_value_get_int (value);
       break;
-    case ARG_SMOOTHING:
+#if ENABLE_SMOOTHING
+    case PROP_SMOOTHING:
       jpegenc->smoothing = g_value_get_int (value);
       break;
+#endif
+    case PROP_IDCT_METHOD:
+      jpegenc->idct_method = g_value_get_enum (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -537,12 +603,17 @@ gst_jpegenc_get_property (GObject * object, guint prop_id, GValue * value,
   GST_OBJECT_LOCK (jpegenc);
 
   switch (prop_id) {
-    case ARG_QUALITY:
+    case PROP_QUALITY:
       g_value_set_int (value, jpegenc->quality);
       break;
-    case ARG_SMOOTHING:
+#if ENABLE_SMOOTHING
+    case PROP_SMOOTHING:
       g_value_set_int (value, jpegenc->smoothing);
       break;
+#endif
+    case PROP_IDCT_METHOD:
+      g_value_set_enum (value, jpegenc->idct_method);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
