@@ -91,6 +91,16 @@ gst_pulse_fill_sample_spec (GstRingBufferSpec * spec, pa_sample_spec * ss)
     ss->format = PA_SAMPLE_S32LE;
   else if (spec->format == GST_S32_BE && spec->width == 32)
     ss->format = PA_SAMPLE_S32BE;
+#if HAVE_PULSE_0_9_15
+  else if (spec->format == GST_S24_3LE && spec->width == 24)
+    ss->format = PA_SAMPLE_S24LE;
+  else if (spec->format == GST_S24_3BE && spec->width == 24)
+    ss->format = PA_SAMPLE_S24BE;
+  else if (spec->format == GST_S24_LE && spec->width == 32)
+    ss->format = PA_SAMPLE_S24_32LE;
+  else if (spec->format == GST_S24_BE && spec->width == 32)
+    ss->format = PA_SAMPLE_S24_32BE;
+#endif
   else
     return FALSE;
 
@@ -123,7 +133,6 @@ gst_pulse_gst_to_channel_map (pa_channel_map * map,
     const GstRingBufferSpec * spec)
 {
   int i;
-
   GstAudioChannelPosition *pos;
 
   pa_channel_map_init (map);
@@ -131,14 +140,10 @@ gst_pulse_gst_to_channel_map (pa_channel_map * map,
   if (!(pos =
           gst_audio_get_channel_positions (gst_caps_get_structure (spec->caps,
                   0)))) {
-/*         g_debug("%s: No channel positions!\n", G_STRFUNC); */
     return NULL;
   }
 
-/*     g_debug("%s: Got channel positions:\n", G_STRFUNC); */
-
   for (i = 0; i < spec->channels; i++) {
-
     if (pos[i] == GST_AUDIO_CHANNEL_POSITION_NONE) {
       /* no valid mappings for these channels */
       g_free (pos);
@@ -147,15 +152,12 @@ gst_pulse_gst_to_channel_map (pa_channel_map * map,
       map->map[i] = gst_pos_to_pa[pos[i]];
     else
       map->map[i] = PA_CHANNEL_POSITION_INVALID;
-
-    /*g_debug("  channel %d: gst: %d pulse: %d\n", i, pos[i], map->map[i]); */
   }
 
   g_free (pos);
   map->channels = spec->channels;
 
   if (!pa_channel_map_valid (map)) {
-/*         g_debug("generated invalid map!\n"); */
     return NULL;
   }
 
