@@ -148,7 +148,9 @@ gst_speex_dec_reset (GstSpeexDec * dec)
   dec->frame_size = 0;
   dec->frame_duration = 0;
   dec->mode = NULL;
-  dec->header = NULL;           /* FIXME: free ?! */
+  free (dec->header);
+  dec->header = NULL;
+  speex_bits_destroy (&dec->bits);
   if (dec->state) {
     speex_decoder_destroy (dec->state);
     dec->state = NULL;
@@ -218,7 +220,7 @@ speex_dec_convert (GstPad * pad,
     case GST_FORMAT_TIME:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
-          scale = sizeof (float) * dec->header->nb_channels;
+          scale = 2 * dec->header->nb_channels;
         case GST_FORMAT_DEFAULT:
           *dest_value =
               gst_util_uint64_scale_int (scale * src_value, dec->header->rate,
@@ -232,7 +234,7 @@ speex_dec_convert (GstPad * pad,
     case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
-          *dest_value = src_value * sizeof (float) * dec->header->nb_channels;
+          *dest_value = src_value * 2 * dec->header->nb_channels;
           break;
         case GST_FORMAT_TIME:
           *dest_value =
@@ -247,11 +249,11 @@ speex_dec_convert (GstPad * pad,
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
         case GST_FORMAT_DEFAULT:
-          *dest_value = src_value / (sizeof (float) * dec->header->nb_channels);
+          *dest_value = src_value / (2 * dec->header->nb_channels);
           break;
         case GST_FORMAT_TIME:
           *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
-              dec->header->rate * sizeof (float) * dec->header->nb_channels);
+              dec->header->rate * 2 * dec->header->nb_channels);
           break;
         default:
           res = FALSE;
@@ -535,6 +537,7 @@ speex_dec_chain_parse_header (GstSpeexDec * dec, GstBuffer * buf)
     dec->callback.callback_id = SPEEX_INBAND_STEREO;
     dec->callback.func = speex_std_stereo_request_handler;
     dec->callback.data = &dec->stereo;
+    dec->stereo = (SpeexStereoState) SPEEX_STEREO_STATE_INIT;
     speex_decoder_ctl (dec->state, SPEEX_SET_HANDLER, &dec->callback);
   }
 
@@ -655,7 +658,7 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
     /* send data to the bitstream */
     speex_bits_read_from (&dec->bits, (char *) data, size);
 
-    fpp = dec->header->frames_per_packet;
+    fpp = 0;
     bits = &dec->bits;
 
     GST_DEBUG_OBJECT (dec, "received buffer of size %u, fpp %d", size, fpp);
@@ -677,8 +680,9 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
   }
 
 
-  /* now decode each frame */
-  for (i = 0; i < fpp; i++) {
+  /* now decode each frame, catering for unknown number of them (e.g. rtp) */
+  for (i = 0; (!fpp || i < fpp) && (!bits || speex_bits_remaining (bits) > 0);
+      i++) {
     GstBuffer *outbuf;
     gint16 *out_data;
     gint ret;
