@@ -174,6 +174,7 @@ static GstStaticPadTemplate audio_sink_factory =
         "rate = (int) [ 1000, 96000 ], " "channels = (int) [ 1, 2 ]; "
         "audio/mpeg, "
         "mpegversion = (int) 4, "
+        "stream-format = (string) raw, "
         "rate = (int) [ 1000, 96000 ], " "channels = (int) [ 1, 2 ]; "
 /*#if 0 VC6 doesn't support #if here ...
         "audio/x-vorbis, "
@@ -342,6 +343,8 @@ gst_avi_mux_pad_reset (GstAviPad * avipad, gboolean free)
   } else {
     GstAviAudioPad *audpad = (GstAviAudioPad *) avipad;
 
+    audpad->samples = 0;
+
     avipad->hdr.type = GST_MAKE_FOURCC ('a', 'u', 'd', 's');
     if (audpad->auds_codec_data) {
       gst_buffer_unref (audpad->auds_codec_data);
@@ -845,8 +848,21 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
         case 4:
         {
           GstBuffer *codec_data_buf = avipad->auds_codec_data;
+          const gchar *stream_format;
           guint codec;
 
+          stream_format = gst_structure_get_string (structure, "stream-format");
+          if (stream_format) {
+            if (strcmp (stream_format, "raw") != 0) {
+              GST_WARNING_OBJECT (avimux, "AAC's stream format '%s' is not "
+                  "supported, please use 'raw'", stream_format);
+              break;
+            }
+          } else {
+            GST_WARNING_OBJECT (avimux, "AAC's stream-format not specified, "
+                "assuming 'raw'");
+          }
+
           /* vbr case needs some special handling */
           if (!codec_data_buf || GST_BUFFER_SIZE (codec_data_buf) < 2) {
             GST_WARNING_OBJECT (avimux, "no (valid) codec_data for AAC audio");
@@ -1008,7 +1024,7 @@ too_late:
   }
 wrong_template:
   {
-    g_warning ("avimuxx: this is not our template!\n");
+    g_warning ("avimux: this is not our template!\n");
     return NULL;
   }
 }
@@ -1467,8 +1483,8 @@ gst_avi_mux_riff_get_header (GstAviPad * avipad, guint32 video_frame_size)
 
 /* write an odml index chunk in the movi list */
 static GstFlowReturn
-gst_avi_mux_write_avix_index (GstAviMux * avimux, gchar * code,
-    gchar * chunk, gst_avi_superindex_entry * super_index,
+gst_avi_mux_write_avix_index (GstAviMux * avimux, GstAviPad * avipad,
+    gchar * code, gchar * chunk, gst_avi_superindex_entry * super_index,
     gint * super_index_count)
 {
   GstFlowReturn res;
@@ -1477,6 +1493,17 @@ gst_avi_mux_write_avix_index (GstAviMux * avimux, gchar * code,
   gst_riff_index_entry *entry;
   gint i;
   guint32 size, entry_count;
+  gboolean is_pcm = FALSE;
+  guint32 pcm_samples = 0;
+
+  /* check if it is pcm */
+  if (avipad && !avipad->is_video) {
+    GstAviAudioPad *audiopad = (GstAviAudioPad *) avipad;
+    if (audiopad->auds.format == GST_RIFF_WAVE_FORMAT_PCM) {
+      pcm_samples = audiopad->samples;
+      is_pcm = TRUE;
+    }
+  }
 
   /* allocate the maximum possible */
   buffer = gst_buffer_new_and_alloc (32 + 8 * avimux->idx_index);
@@ -1528,7 +1555,11 @@ gst_avi_mux_write_avix_index (GstAviMux * avimux, gchar * code,
     i = *super_index_count;
     super_index[i].offset = GUINT64_TO_LE (avimux->total_data);
     super_index[i].size = GUINT32_TO_LE (size);
-    super_index[i].duration = GUINT32_TO_LE (entry_count);
+    if (is_pcm) {
+      super_index[i].duration = GUINT32_TO_LE (pcm_samples);
+    } else {
+      super_index[i].duration = GUINT32_TO_LE (entry_count);
+    }
     (*super_index_count)++;
   } else
     GST_WARNING_OBJECT (avimux, "No more room in superindex of stream %s",
@@ -1547,15 +1578,26 @@ gst_avi_mux_write_avix_index (GstAviMux * avimux, gchar * code,
 /* some other usable functions (thankyou xawtv ;-) ) */
 
 static void
-gst_avi_mux_add_index (GstAviMux * avimux, gchar * code, guint32 flags,
+gst_avi_mux_add_index (GstAviMux * avimux, GstAviPad * avipad, guint32 flags,
     guint32 size)
 {
+  gchar *code = avipad->tag;
   if (avimux->idx_index == avimux->idx_count) {
     avimux->idx_count += 256;
     avimux->idx =
         g_realloc (avimux->idx,
         avimux->idx_count * sizeof (gst_riff_index_entry));
   }
+
+  /* in case of pcm audio, we need to count the number of samples for
+   * putting in the indx entries */
+  if (!avipad->is_video) {
+    GstAviAudioPad *audiopad = (GstAviAudioPad *) avipad;
+    if (audiopad->auds.format == GST_RIFF_WAVE_FORMAT_PCM) {
+      audiopad->samples += size / audiopad->auds.blockalign;
+    }
+  }
+
   memcpy (&(avimux->idx[avimux->idx_index].id), code, 4);
   avimux->idx[avimux->idx_index].flags = GUINT32_TO_LE (flags);
   avimux->idx[avimux->idx_index].offset = GUINT32_TO_LE (avimux->idx_offset);
@@ -1615,7 +1657,7 @@ gst_avi_mux_bigfile (GstAviMux * avimux, gboolean last)
 
     node = node->next;
 
-    res = gst_avi_mux_write_avix_index (avimux, avipad->tag,
+    res = gst_avi_mux_write_avix_index (avimux, avipad, avipad->tag,
         avipad->idx_tag, avipad->idx, &avipad->idx_index);
     if (res != GST_FLOW_OK)
       return res;
@@ -1657,6 +1699,15 @@ gst_avi_mux_bigfile (GstAviMux * avimux, gboolean last)
   avimux->numx_frames = 0;
   avimux->datax_size = 4;       /* movi tag */
   avimux->idx_index = 0;
+  node = avimux->sinkpads;
+  while (node) {
+    GstAviPad *avipad = (GstAviPad *) node->data;
+    node = node->next;
+    if (!avipad->is_video) {
+      GstAviAudioPad *audiopad = (GstAviAudioPad *) avipad;
+      audiopad->samples = 0;
+    }
+  }
 
   header = gst_avi_mux_riff_get_avix_header (0);
   avimux->total_data += GST_BUFFER_SIZE (header);
@@ -1922,11 +1973,11 @@ gst_avi_mux_do_buffer (GstAviMux * avimux, GstAviPad * avipad)
   if (G_UNLIKELY (avipad->hook))
     avipad->hook (avimux, avipad, data);
 
-  if (avipad->is_video) {
-    /* the suggested buffer size is the max frame size */
-    if (avipad->hdr.bufsize < GST_BUFFER_SIZE (data))
-      avipad->hdr.bufsize = GST_BUFFER_SIZE (data);
+  /* the suggested buffer size is the max frame size */
+  if (avipad->hdr.bufsize < GST_BUFFER_SIZE (data))
+    avipad->hdr.bufsize = GST_BUFFER_SIZE (data);
 
+  if (avipad->is_video) {
     avimux->total_frames++;
 
     if (avimux->is_bigfile) {
@@ -1946,7 +1997,7 @@ gst_avi_mux_do_buffer (GstAviMux * avimux, GstAviPad * avipad)
     audpad->audio_time += GST_BUFFER_DURATION (data);
   }
 
-  gst_avi_mux_add_index (avimux, avipad->tag, flags, GST_BUFFER_SIZE (data));
+  gst_avi_mux_add_index (avimux, avipad, flags, GST_BUFFER_SIZE (data));
 
   /* prepare buffers for sending */
   gst_buffer_set_caps (header, GST_PAD_CAPS (avimux->srcpad));
@@ -1979,7 +2030,7 @@ gst_avi_mux_do_one_buffer (GstAviMux * avimux)
   GstAviPad *avipad, *best_pad;
   GSList *node;
   GstBuffer *buffer;
-  GstClockTime time, best_time;
+  GstClockTime time, best_time, delay;
 
   node = avimux->sinkpads;
   best_pad = NULL;
@@ -1996,12 +2047,14 @@ gst_avi_mux_do_one_buffer (GstAviMux * avimux)
     time = GST_BUFFER_TIMESTAMP (buffer);
     gst_buffer_unref (buffer);
 
+    delay = avipad->is_video ? GST_SECOND / 2 : 0;
+
     /* invalid timestamp buffers pass first,
      * these are probably initialization buffers */
     if (best_pad == NULL || !GST_CLOCK_TIME_IS_VALID (time)
-        || (GST_CLOCK_TIME_IS_VALID (best_time) && time < best_time)) {
+        || (GST_CLOCK_TIME_IS_VALID (best_time) && time + delay < best_time)) {
       best_pad = avipad;
-      best_time = time;
+      best_time = time + delay;
     }
   }
 
