@@ -24,6 +24,7 @@
 #endif
 
 #include <string.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <gst/tag/tag.h>
 #include <gst/base/gsttypefindhelper.h>
@@ -275,7 +276,12 @@ parse_comment_frame (ID3TagsWorking * work)
         g_ascii_isalpha (language[0]) &&
         g_ascii_isalpha (language[1]) &&
         (g_ascii_isalpha (language[2]) || language[2] == '\0')) {
-      s = g_strdup_printf ("%s[%s]=%s", description, language, text);
+      const gchar *lang_code;
+
+      /* prefer two-letter ISO 639-1 code if we have a mapping */
+      lang_code = gst_tag_get_language_code (language);
+      s = g_strdup_printf ("%s[%s]=%s", description,
+          (lang_code) ? lang_code : language, text);
     } else {
       s = g_strdup_printf ("%s=%s", description, text);
     }
@@ -978,7 +984,7 @@ string_utf8_dup (const gchar * start, const guint size)
       if ((utf8 =
               g_convert (start, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
         if (bytes_read == size) {
-          GST_DEBUG ("Using charset %s to interperate id3 tags\n", c);
+          GST_DEBUG ("Using charset %s to interperate id3 tags\n", *c);
           g_strfreev (csets);
           goto beach;
         }
