@@ -22,21 +22,46 @@
 /**
  * SECTION:element-level
  *
- * <refsect2>
- * <para>
- * Level analyses incoming audio buffers and, if the
- * <link linkend="GstLevel--message">message property</link> is #TRUE,
- * generates an element message named
+ * Level analyses incoming audio buffers and, if the #GstLevel:message property
+ * is #TRUE, generates an element message named
  * <classname>&quot;level&quot;</classname>:
- * after each interval of time given by the
- * <link linkend="GstLevel--interval">interval property</link>.
- * The message's structure contains four fields:
+ * after each interval of time given by the #GstLevel:interval property.
+ * The message's structure contains these fields:
  * <itemizedlist>
  * <listitem>
  *   <para>
  *   #GstClockTime
+ *   <classname>&quot;timestamp&quot;</classname>:
+ *   the timestamp of the buffer that triggered the message.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;stream-time&quot;</classname>:
+ *   the stream time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;running-time&quot;</classname>:
+ *   the running_time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;duration&quot;</classname>:
+ *   the duration of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
  *   <classname>&quot;endtime&quot;</classname>:
- *   the end time of the buffer that triggered the message
+ *   the end time of the buffer that triggered the message as stream time (this
+ *   is deprecated, as it can be calculated from stream-time + duration)
  *   </para>
  * </listitem>
  * <listitem>
@@ -66,12 +91,13 @@
  *   the Root Mean Square (or average power) level in dB for each channel
  *   </para>
  * </listitem>
-  * </itemizedlist>
- * </para>
+ * </itemizedlist>
+ *
+ * <refsect2>
  * <title>Example application</title>
- * <para>
- * <include xmlns="http://www.w3.org/2003/XInclude" href="element-level-example.xml" />
- * </para>
+ * |[
+ * <xi:include xmlns:xi="http://www.w3.org/2003/XInclude" parse="text" href="../../../../tests/examples/level/level-example.c" />
+ * ]|
  * </refsect2>
  */
 
@@ -79,10 +105,12 @@
 #include "config.h"
 #endif
 #include <string.h>
+#include <math.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
+/*#include <liboil/liboil.h>*/
+
 #include "gstlevel.h"
-#include "math.h"
 
 GST_DEBUG_CATEGORY_STATIC (level_debug);
 #define GST_CAT_DEFAULT level_debug
@@ -100,8 +128,8 @@ static GstStaticPadTemplate sink_template_factory =
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, "
+        "width = (int) { 8, 16, 32 }, "
+        "depth = (int) { 8, 16, 32 }, "
         "signed = (boolean) true; "
         "audio/x-raw-float, "
         "rate = (int) [ 1, MAX ], "
@@ -117,8 +145,8 @@ static GstStaticPadTemplate src_template_factory =
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, "
+        "width = (int) { 8, 16, 32 }, "
+        "depth = (int) { 8, 16, 32 }, "
         "signed = (boolean) true; "
         "audio/x-raw-float, "
         "rate = (int) [ 1, MAX ], "
@@ -142,12 +170,11 @@ static void gst_level_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_level_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static void gst_level_dispose (GObject * obj);
+static void gst_level_finalize (GObject * obj);
 
 static gboolean gst_level_set_caps (GstBaseTransform * trans, GstCaps * in,
     GstCaps * out);
 static gboolean gst_level_start (GstBaseTransform * trans);
-static gboolean gst_level_event (GstBaseTransform * trans, GstEvent * event);
 static GstFlowReturn gst_level_transform_ip (GstBaseTransform * trans,
     GstBuffer * in);
 
@@ -167,29 +194,26 @@ gst_level_base_init (gpointer g_class)
 static void
 gst_level_class_init (GstLevelClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstBaseTransformClass *trans_class;
-
-  gobject_class = (GObjectClass *) klass;
-  trans_class = (GstBaseTransformClass *) klass;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstBaseTransformClass *trans_class = GST_BASE_TRANSFORM_CLASS (klass);
 
   gobject_class->set_property = gst_level_set_property;
   gobject_class->get_property = gst_level_get_property;
-  gobject_class->dispose = gst_level_dispose;
+  gobject_class->finalize = gst_level_finalize;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIGNAL_LEVEL,
-      g_param_spec_boolean ("message", "mesage",
+  g_object_class_install_property (gobject_class, PROP_SIGNAL_LEVEL,
+      g_param_spec_boolean ("message", "message",
           "Post a level message for each passed interval",
           TRUE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIGNAL_INTERVAL,
+  g_object_class_install_property (gobject_class, PROP_SIGNAL_INTERVAL,
       g_param_spec_uint64 ("interval", "Interval",
           "Interval of time between message posts (in nanoseconds)",
           1, G_MAXUINT64, GST_SECOND / 10, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PEAK_TTL,
+  g_object_class_install_property (gobject_class, PROP_PEAK_TTL,
       g_param_spec_uint64 ("peak-ttl", "Peak TTL",
           "Time To Live of decay peak before it falls back (in nanoseconds)",
           0, G_MAXUINT64, GST_SECOND / 10 * 3, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PEAK_FALLOFF,
+  g_object_class_install_property (gobject_class, PROP_PEAK_FALLOFF,
       g_param_spec_double ("peak-falloff", "Peak Falloff",
           "Decay rate of decay peak after TTL (in dB/sec)",
           0.0, G_MAXDOUBLE, 10.0, G_PARAM_READWRITE));
@@ -198,7 +222,6 @@ gst_level_class_init (GstLevelClass * klass)
 
   trans_class->set_caps = GST_DEBUG_FUNCPTR (gst_level_set_caps);
   trans_class->start = GST_DEBUG_FUNCPTR (gst_level_start);
-  trans_class->event = GST_DEBUG_FUNCPTR (gst_level_event);
   trans_class->transform_ip = GST_DEBUG_FUNCPTR (gst_level_transform_ip);
   trans_class->passthrough_on_same_caps = TRUE;
 }
@@ -220,10 +243,12 @@ gst_level_init (GstLevel * filter, GstLevelClass * g_class)
   filter->message = TRUE;
 
   filter->process = NULL;
+
+  gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (filter), TRUE);
 }
 
 static void
-gst_level_dispose (GObject * obj)
+gst_level_finalize (GObject * obj)
 {
   GstLevel *filter = GST_LEVEL (obj);
 
@@ -241,7 +266,7 @@ gst_level_dispose (GObject * obj)
   filter->decay_peak_base = NULL;
   filter->decay_peak_age = NULL;
 
-  G_OBJECT_CLASS (parent_class)->dispose (obj);
+  G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -256,6 +281,10 @@ gst_level_set_property (GObject * object, guint prop_id,
       break;
     case PROP_SIGNAL_INTERVAL:
       filter->interval = g_value_get_uint64 (value);
+      if (filter->rate) {
+        filter->interval_frames =
+            GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate);
+      }
       break;
     case PROP_PEAK_TTL:
       filter->decay_peak_ttl =
@@ -265,6 +294,7 @@ gst_level_set_property (GObject * object, guint prop_id,
       filter->decay_peak_falloff = g_value_get_double (value);
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
@@ -323,11 +353,12 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
   register gdouble peaksquare = 0.0; /* Peak Square Sample */                 \
   gdouble normalizer;               /* divisor to get a [-1.0, 1.0] range */  \
                                                                               \
-  *NCS = 0.0;                       /* Normalized Cumulative Square */        \
-  *NPS = 0.0;                       /* Normalized Peask Square */             \
+  /* *NCS = 0.0; Normalized Cumulative Square */                              \
+  /* *NPS = 0.0; Normalized Peask Square */                                   \
                                                                               \
-  normalizer = (gdouble) (1 << (RESOLUTION * 2));                             \
+  normalizer = (gdouble) (G_GINT64_CONSTANT(1) << (RESOLUTION * 2));          \
                                                                               \
+  /* oil_squaresum_shifted_s16(&squaresum,in,num); */                         \
   for (j = 0; j < num; j += channels)                                         \
   {                                                                           \
     square = ((gdouble) in[j]) * in[j];                                       \
@@ -339,6 +370,7 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
   *NPS = peaksquare / normalizer;                                             \
 }
 
+DEFINE_INT_LEVEL_CALCULATOR (gint32, 31);
 DEFINE_INT_LEVEL_CALCULATOR (gint16, 15);
 DEFINE_INT_LEVEL_CALCULATOR (gint8, 7);
 
@@ -353,9 +385,10 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
   register gdouble square = 0.0;     /* Square */                             \
   register gdouble peaksquare = 0.0; /* Peak Square Sample */                 \
                                                                               \
-  *NCS = 0.0;                       /* Normalized Cumulative Square */        \
-  *NPS = 0.0;                       /* Normalized Peask Square */             \
+  /* *NCS = 0.0; Normalized Cumulative Square */                              \
+  /* *NPS = 0.0; Normalized Peask Square */                                   \
                                                                               \
+  /* oil_squaresum_f64(&squaresum,in,num); */                                 \
   for (j = 0; j < num; j += channels)                                         \
   {                                                                           \
     square = ((gdouble) in[j]) * in[j];                                       \
@@ -370,6 +403,17 @@ gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
 DEFINE_FLOAT_LEVEL_CALCULATOR (gfloat);
 DEFINE_FLOAT_LEVEL_CALCULATOR (gdouble);
 
+/* we would need stride to deinterleave also
+static void inline
+gst_level_calculate_gdouble (gpointer data, guint num, guint channels,
+                            gdouble *NCS, gdouble *NPS)
+{
+  oil_squaresum_f64(NCS,(gdouble *)data,num);
+  *NPS = 0.0;
+}
+*/
+
+
 static gint
 structure_get_int (GstStructure * structure, const gchar * field)
 {
@@ -387,7 +431,7 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   GstLevel *filter = GST_LEVEL (trans);
   const gchar *mimetype;
   GstStructure *structure;
-  int i;
+  gint i;
 
   structure = gst_caps_get_structure (in, 0);
   filter->rate = structure_get_int (structure, "rate");
@@ -406,6 +450,9 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
       case 16:
         filter->process = gst_level_calculate_gint16;
         break;
+      case 32:
+        filter->process = gst_level_calculate_gint32;
+        break;
     }
   } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
     GST_DEBUG_OBJECT (filter, "use float, %u", filter->width);
@@ -426,20 +473,23 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   g_free (filter->decay_peak);
   g_free (filter->decay_peak_base);
   g_free (filter->decay_peak_age);
-  filter->CS = g_new (double, filter->channels);
-  filter->peak = g_new (double, filter->channels);
-  filter->last_peak = g_new (double, filter->channels);
-  filter->decay_peak = g_new (double, filter->channels);
-  filter->decay_peak_base = g_new (double, filter->channels);
+  filter->CS = g_new (gdouble, filter->channels);
+  filter->peak = g_new (gdouble, filter->channels);
+  filter->last_peak = g_new (gdouble, filter->channels);
+  filter->decay_peak = g_new (gdouble, filter->channels);
+  filter->decay_peak_base = g_new (gdouble, filter->channels);
 
   filter->decay_peak_age = g_new (GstClockTime, filter->channels);
 
   for (i = 0; i < filter->channels; ++i) {
     filter->CS[i] = filter->peak[i] = filter->last_peak[i] =
         filter->decay_peak[i] = filter->decay_peak_base[i] = 0.0;
-    filter->decay_peak_age[i] = G_GINT64_CONSTANT (0);
+    filter->decay_peak_age[i] = G_GUINT64_CONSTANT (0);
   }
 
+  filter->interval_frames =
+      GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate);
+
   return TRUE;
 }
 
@@ -449,50 +499,34 @@ gst_level_start (GstBaseTransform * trans)
   GstLevel *filter = GST_LEVEL (trans);
 
   filter->num_frames = 0;
-  gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
-
-  return TRUE;
-}
-
-static gboolean
-gst_level_event (GstBaseTransform * trans, GstEvent * event)
-{
-  GstLevel *filter = GST_LEVEL (trans);
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:{
-      GstFormat format;
-      gdouble rate, arate;
-      gint64 start, stop, time;
-      gboolean update;
-
-      /* the newsegment values are used to clip the input samples
-       * and to convert the incomming timestamps to running time */
-      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
-          &start, &stop, &time);
-
-      /* now configure the values */
-      gst_segment_set_newsegment_full (&filter->segment, update,
-          rate, arate, format, start, stop, time);
-      break;
-    }
-    default:
-      break;
-  }
 
   return TRUE;
 }
 
 static GstMessage *
-gst_level_message_new (GstLevel * l, GstClockTime endtime)
+gst_level_message_new (GstLevel * level, GstClockTime timestamp,
+    GstClockTime duration)
 {
+  GstBaseTransform *trans = GST_BASE_TRANSFORM_CAST (level);
   GstStructure *s;
   GValue v = { 0, };
+  GstClockTime endtime, running_time, stream_time;
 
   g_value_init (&v, GST_TYPE_LIST);
 
-  s = gst_structure_new ("level", "endtime", GST_TYPE_CLOCK_TIME,
-      endtime, NULL);
+  running_time = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  stream_time = gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  /* endtime is for backwards compatibility */
+  endtime = stream_time + duration;
+
+  s = gst_structure_new ("level",
+      "endtime", GST_TYPE_CLOCK_TIME, endtime,
+      "timestamp", G_TYPE_UINT64, timestamp,
+      "stream-time", G_TYPE_UINT64, stream_time,
+      "running-time", G_TYPE_UINT64, running_time,
+      "duration", G_TYPE_UINT64, duration, NULL);
   /* will copy-by-value */
   gst_structure_set_value (s, "rms", &v);
   gst_structure_set_value (s, "peak", &v);
@@ -500,7 +534,7 @@ gst_level_message_new (GstLevel * l, GstClockTime endtime)
 
   g_value_unset (&v);
 
-  return gst_message_new_element (GST_OBJECT (l), s);
+  return gst_message_new_element (GST_OBJECT (level), s);
 }
 
 static void
@@ -535,39 +569,39 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 {
   GstLevel *filter;
   guint8 *in_data;
-  double CS = 0.0;
+  gdouble CS;
+  guint i;
   guint num_frames = 0;
   guint num_int_samples = 0;    /* number of interleaved samples
                                  * ie. total count for all channels combined */
-  guint i;
+  GstClockTimeDiff falloff_time;
 
   filter = GST_LEVEL (trans);
 
-  for (i = 0; i < filter->channels; ++i)
-    filter->peak[i] = 0.0;
-
   in_data = GST_BUFFER_DATA (in);
   num_int_samples = GST_BUFFER_SIZE (in) / (filter->width / 8);
 
+  GST_LOG_OBJECT (filter, "analyzing %u sample frames at ts %" GST_TIME_FORMAT,
+      num_int_samples, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (in)));
+
   g_return_val_if_fail (num_int_samples % filter->channels == 0,
       GST_FLOW_ERROR);
 
   num_frames = num_int_samples / filter->channels;
 
   for (i = 0; i < filter->channels; ++i) {
-    CS = 0.0;
-    filter->process (in_data, num_int_samples, filter->channels, &CS,
-        &filter->peak[i]);
-    GST_LOG_OBJECT (filter,
-        "channel %d, cumulative sum %f, peak %f, over %d samples/%d channels",
-        i, CS, filter->peak[i], num_int_samples, filter->channels);
-    filter->CS[i] += CS;
+    if (!GST_BUFFER_FLAG_IS_SET (in, GST_BUFFER_FLAG_GAP)) {
+      filter->process (in_data, num_int_samples, filter->channels, &CS,
+          &filter->peak[i]);
+      GST_LOG_OBJECT (filter,
+          "channel %d, cumulative sum %f, peak %f, over %d samples/%d channels",
+          i, CS, filter->peak[i], num_int_samples, filter->channels);
+      filter->CS[i] += CS;
+    } else {
+      filter->peak[i] = 0.0;
+    }
     in_data += (filter->width / 8);
-  }
 
-  filter->num_frames += num_frames;
-
-  for (i = 0; i < filter->channels; ++i) {
     filter->decay_peak_age[i] +=
         GST_FRAMES_TO_CLOCK_TIME (num_frames, filter->rate);
     GST_LOG_OBJECT (filter, "filter peak info [%d]: decay peak %f, age %"
@@ -579,16 +613,15 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
       filter->last_peak[i] = filter->peak[i];
 
     /* make decay peak fall off if too old */
-    if (gst_guint64_to_gdouble (filter->decay_peak_age[i]) >
-        filter->decay_peak_ttl) {
-      double falloff_dB;
-      double falloff;
-      GstClockTimeDiff falloff_time;
-      double length;            /* length of falloff time in seconds */
+    falloff_time =
+        GST_CLOCK_DIFF (gst_gdouble_to_guint64 (filter->decay_peak_ttl),
+        filter->decay_peak_age[i]);
+    if (falloff_time > 0) {
+      gdouble falloff_dB;
+      gdouble falloff;
+      gdouble length;           /* length of falloff time in seconds */
 
-      falloff_time = GST_CLOCK_DIFF (filter->decay_peak_ttl,
-          gst_guint64_to_gdouble (filter->decay_peak_age[i]));
-      length = (gdouble) falloff_time / GST_SECOND;
+      length = (gdouble) falloff_time / (gdouble) GST_SECOND;
       falloff_dB = filter->decay_peak_falloff * length;
       falloff = pow (10, falloff_dB / -20.0);
 
@@ -615,25 +648,28 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
     }
   }
 
+  if (G_UNLIKELY (!filter->num_frames)) {
+    /* remember start timestamp for message */
+    filter->message_ts = GST_BUFFER_TIMESTAMP (in);
+  }
+  filter->num_frames += num_frames;
+
   /* do we need to message ? */
-  if (filter->num_frames >=
-      GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate)) {
+  if (filter->num_frames >= filter->interval_frames) {
     if (filter->message) {
       GstMessage *m;
-      GstClockTime endtime =
-          gst_segment_to_running_time (&filter->segment, GST_FORMAT_TIME,
-          GST_BUFFER_TIMESTAMP (in))
-          + GST_FRAMES_TO_CLOCK_TIME (num_frames, filter->rate);
+      GstClockTime duration =
+          GST_FRAMES_TO_CLOCK_TIME (filter->num_frames, filter->rate);
 
-      m = gst_level_message_new (filter, endtime);
+      m = gst_level_message_new (filter, filter->message_ts, duration);
 
       GST_LOG_OBJECT (filter,
-          "message: end time %" GST_TIME_FORMAT ", num_frames %d",
-          GST_TIME_ARGS (endtime), filter->num_frames);
+          "message: ts %" GST_TIME_FORMAT ", num_frames %d",
+          GST_TIME_ARGS (filter->message_ts), filter->num_frames);
 
       for (i = 0; i < filter->channels; ++i) {
-        double RMS;
-        double RMSdB, lastdB, decaydB;
+        gdouble RMS;
+        gdouble RMSdB, lastdB, decaydB;
 
         RMS = sqrt (filter->CS[i] / filter->num_frames);
         GST_LOG_OBJECT (filter,
@@ -678,6 +714,8 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  /*oil_init (); */
+
   return gst_element_register (plugin, "level", GST_RANK_NONE, GST_TYPE_LEVEL);
 }
 
