@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,8 +21,7 @@
 #define __GST_RTP_L16_PAY_H__
 
 #include <gst/gst.h>
-#include <gst/rtp/gstbasertppayload.h>
-#include <gst/base/gstadapter.h>
+#include <gst/rtp/gstbasertpaudiopayload.h>
 
 G_BEGIN_DECLS
 
@@ -42,10 +41,7 @@ typedef struct _GstRtpL16PayClass GstRtpL16PayClass;
 
 struct _GstRtpL16Pay
 {
-  GstBaseRTPPayload payload;
-
-  GstAdapter  *adapter;
-  GstClockTime first_ts;
+  GstBaseRTPAudioPayload payload;
 
   gint rate;
   gint channels;
@@ -53,7 +49,7 @@ struct _GstRtpL16Pay
 
 struct _GstRtpL16PayClass
 {
-  GstBaseRTPPayloadClass parent_class;
+  GstBaseRTPAudioPayloadClass parent_class;
 };
 
 gboolean gst_rtp_L16_pay_plugin_init (GstPlugin * plugin);
