@@ -65,9 +65,6 @@ static gboolean gst_rtp_mpa_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_mpa_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static GstStateChangeReturn gst_rtp_mpa_depay_change_state (GstElement *
-    element, GstStateChange transition);
-
 static void
 gst_rtp_mpa_depay_base_init (gpointer klass)
 {
@@ -84,18 +81,10 @@ gst_rtp_mpa_depay_base_init (gpointer klass)
 static void
 gst_rtp_mpa_depay_class_init (GstRtpMPADepayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
-  gstelement_class->change_state = gst_rtp_mpa_depay_change_state;
-
   gstbasertpdepayload_class->set_caps = gst_rtp_mpa_depay_setcaps;
   gstbasertpdepayload_class->process = gst_rtp_mpa_depay_process;
 
@@ -114,13 +103,10 @@ static gboolean
 gst_rtp_mpa_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstStructure *structure;
-  GstRtpMPADepay *rtpmpadepay;
   GstCaps *outcaps;
   gint clock_rate;
   gboolean res;
 
-  rtpmpadepay = GST_RTP_MPA_DEPAY (depayload);
-
   structure = gst_caps_get_structure (caps, 0);
 
   if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
@@ -193,32 +179,6 @@ empty_packet:
   }
 }
 
-static GstStateChangeReturn
-gst_rtp_mpa_depay_change_state (GstElement * element, GstStateChange transition)
-{
-  GstRtpMPADepay *rtpmpadepay;
-  GstStateChangeReturn ret;
-
-  rtpmpadepay = GST_RTP_MPA_DEPAY (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-  return ret;
-}
-
 gboolean
 gst_rtp_mpa_depay_plugin_init (GstPlugin * plugin)
 {
