@@ -1,8 +1,8 @@
 /*
  * GStreamer
- * Copyright 2005,2006 Zaheer Abbas Merali  <zaheerabbas at merali dot org>
- * Copyright 2007 Pioneers of the Inevitable <songbird@songbirdnest.com>
- * 
+ * Copyright (C) 2005,2006 Zaheer Abbas Merali <zaheerabbas at merali dot org>
+ * Copyright (C) 2007,2008 Pioneers of the Inevitable <songbird@songbirdnest.com>
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -41,22 +41,24 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- *  The development of this code was made possible due to the involvement of Pioneers of i
- *  the Inevitable, the creators of the Songbird Music player
+ * The development of this code was made possible due to the involvement of
+ * Pioneers of the Inevitable, the creators of the Songbird Music player
  *
  */
 
 /**
- * SECTION:element-plugin
+ * SECTION:element-osxaudiosink
+ *
+ * This element renders raw audio samples using the CoreAudio api.
  *
  * <refsect2>
- * <title>Example launch line</title>
- * <para>
- * <programlisting>
- * gst-launch -v -m audiotestsrc ! audioconvert ! osxaudiosink
- * </programlisting>
- * </para>
+ * <title>Example pipelines</title>
+ * |[
+ * gst-launch filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! osxaudiosink
+ * ]| Play an Ogg/Vorbis file.
  * </refsect2>
+ *
+ * Last reviewed on 2006-03-01 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,9 +67,8 @@
 
 #include <gst/gst.h>
 #include <CoreAudio/CoreAudio.h>
+#include <CoreAudio/AudioHardware.h>
 #include "gstosxaudiosink.h"
-#include "gstosxaudiosrc.h"
-
 #include "gstosxaudioelement.h"
 
 GST_DEBUG_CATEGORY_STATIC (osx_audiosink_debug);
@@ -89,9 +90,12 @@ enum
 enum
 {
   ARG_0,
-  ARG_DEVICE
+  ARG_DEVICE,
+  ARG_VOLUME
 };
 
+#define DEFAULT_VOLUME 1.0
+
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -99,27 +103,29 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
         "endianness = (int) {" G_STRINGIFY (G_BYTE_ORDER) " }, "
         "signed = (boolean) { TRUE }, "
         "width = (int) 32, "
-        "depth = (int) 32, " "rate = (int) 44100, " "channels = (int) 2")
+        "depth = (int) 32, "
+        "rate = (int) [1, MAX], " "channels = (int) [1, MAX]")
     );
 
 static void gst_osx_audio_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_osx_audio_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static GstCaps *gst_osx_audio_sink_getcaps (GstBaseSink * sink);
-
 
 static GstRingBuffer *gst_osx_audio_sink_create_ringbuffer (GstBaseAudioSink *
     sink);
-/*static GstCaps* gst_osx_audio_sink_getcaps (GstBaseSink * bsink);*/
 static void gst_osx_audio_sink_osxelement_init (gpointer g_iface,
     gpointer iface_data);
-OSStatus gst_osx_audio_sink_io_proc (AudioDeviceID inDevice,
-    const AudioTimeStamp * inNow, const AudioBufferList * inInputData,
-    const AudioTimeStamp * inInputTime, AudioBufferList * outOutputData,
-    const AudioTimeStamp * inOutputTime, void *inClientData);
+static void gst_osx_audio_sink_select_device (GstOsxAudioSink * osxsink);
+static void gst_osx_audio_sink_set_volume (GstOsxAudioSink * sink);
+
+static OSStatus gst_osx_audio_sink_io_proc (GstOsxRingBuffer * buf,
+    AudioUnitRenderActionFlags * ioActionFlags,
+    const AudioTimeStamp * inTimeStamp,
+    UInt32 inBusNumber, UInt32 inNumberFrames, AudioBufferList * bufferList);
+
 static void
-gst_osx_audio_sink_osxelement_do_init (GType type)
+gst_osx_audio_sink_do_init (GType type)
 {
   static const GInterfaceInfo osxelement_info = {
     gst_osx_audio_sink_osxelement_init,
@@ -135,8 +141,7 @@ gst_osx_audio_sink_osxelement_do_init (GType type)
 }
 
 GST_BOILERPLATE_FULL (GstOsxAudioSink, gst_osx_audio_sink, GstBaseAudioSink,
-    GST_TYPE_BASE_AUDIO_SINK, gst_osx_audio_sink_osxelement_do_init);
-
+    GST_TYPE_BASE_AUDIO_SINK, gst_osx_audio_sink_do_init);
 
 static void
 gst_osx_audio_sink_base_init (gpointer g_class)
@@ -149,7 +154,6 @@ gst_osx_audio_sink_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_osx_audio_sink_details);
 }
 
-/* initialize the plugin's class */
 static void
 gst_osx_audio_sink_class_init (GstOsxAudioSinkClass * klass)
 {
@@ -174,25 +178,21 @@ gst_osx_audio_sink_class_init (GstOsxAudioSinkClass * klass)
       g_param_spec_int ("device", "Device ID", "Device ID of output device",
           0, G_MAXINT, 0, G_PARAM_READWRITE));
 
-  gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_osx_audio_sink_getcaps);
+  g_object_class_install_property (gobject_class, ARG_VOLUME,
+      g_param_spec_double ("volume", "Volume", "Volume of this stream",
+          0, 1.0, 1.0, G_PARAM_READWRITE));
+
   gstbaseaudiosink_class->create_ringbuffer =
       GST_DEBUG_FUNCPTR (gst_osx_audio_sink_create_ringbuffer);
-
 }
 
-/* initialize the new element
- * instantiate pads and add them to element
- * set functions
- * initialize structure
- */
 static void
 gst_osx_audio_sink_init (GstOsxAudioSink * sink, GstOsxAudioSinkClass * gclass)
 {
-/*  GstElementClass *klass = GST_ELEMENT_GET_CLASS (sink); */
-  sink->ringbuffer = NULL;
   GST_DEBUG ("Initialising object");
 
-  gst_osx_audio_sink_create_ringbuffer (GST_BASE_AUDIO_SINK (sink));
+  sink->device_id = kAudioDeviceUnknown;
+  sink->volume = DEFAULT_VOLUME;
 }
 
 static void
@@ -203,8 +203,11 @@ gst_osx_audio_sink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_DEVICE:
-      if (sink->ringbuffer)
-        sink->ringbuffer->device_id = g_value_get_int (value);
+      sink->device_id = g_value_get_int (value);
+      break;
+    case ARG_VOLUME:
+      sink->volume = g_value_get_double (value);
+      gst_osx_audio_sink_set_volume (sink);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -217,14 +220,12 @@ gst_osx_audio_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   GstOsxAudioSink *sink = GST_OSX_AUDIO_SINK (object);
-  int val = 0;
-
   switch (prop_id) {
     case ARG_DEVICE:
-      if (sink->ringbuffer)
-        val = sink->ringbuffer->device_id;
-
-      g_value_set_int (value, val);
+      g_value_set_int (value, sink->device_id);
+      break;
+    case ARG_VOLUME:
+      g_value_set_double (value, sink->volume);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -232,83 +233,72 @@ gst_osx_audio_sink_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* GstElement vmethod implementations */
-
-/* GstBaseSink vmethod implementations */
-static GstCaps *
-gst_osx_audio_sink_getcaps (GstBaseSink * sink)
-{
-  GstCaps *caps;
-  GstOsxAudioSink *osxsink;
-  OSStatus status;
-  AudioValueRange rates[10];
-  UInt32 propertySize;
-  int i;
-
-  propertySize = sizeof (AudioValueRange) * 9;
-  osxsink = GST_OSX_AUDIO_SINK (sink);
-
-  caps = gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SINK_PAD
-          (sink)));
-
-
-  status = AudioDeviceGetProperty (osxsink->ringbuffer->device_id, 0, FALSE,
-      kAudioDevicePropertyAvailableNominalSampleRates, &propertySize, &rates);
-
-  GST_DEBUG
-      ("Getting available sample rates: Status: %ld number of ranges: %lu",
-      status, propertySize / sizeof (AudioValueRange));
-
-  for (i = 0; i < propertySize / sizeof (AudioValueRange); i++) {
-    GST_LOG_OBJECT (osxsink, "Range from %f to %f", rates[i].mMinimum,
-        rates[i].mMaximum);
-  }
-
-  return caps;
-}
-
-/* GstBaseAudioSink vmethod implementations */
 static GstRingBuffer *
 gst_osx_audio_sink_create_ringbuffer (GstBaseAudioSink * sink)
 {
   GstOsxAudioSink *osxsink;
+  GstOsxRingBuffer *ringbuffer;
 
   osxsink = GST_OSX_AUDIO_SINK (sink);
-  if (!osxsink->ringbuffer) {
-    GST_DEBUG ("Creating ringbuffer");
-    osxsink->ringbuffer = g_object_new (GST_TYPE_OSX_RING_BUFFER, NULL);
-    GST_DEBUG ("osx sink 0x%p element 0x%p  ioproc 0x%p", osxsink,
-        GST_OSX_AUDIO_ELEMENT_GET_INTERFACE (osxsink),
-        (void *) gst_osx_audio_sink_io_proc);
-    osxsink->ringbuffer->element =
-        GST_OSX_AUDIO_ELEMENT_GET_INTERFACE (osxsink);
-  }
 
-  return GST_RING_BUFFER (osxsink->ringbuffer);
+  gst_osx_audio_sink_select_device (osxsink);
+
+  GST_DEBUG ("Creating ringbuffer");
+  ringbuffer = g_object_new (GST_TYPE_OSX_RING_BUFFER, NULL);
+  GST_DEBUG ("osx sink 0x%p element 0x%p  ioproc 0x%p", osxsink,
+      GST_OSX_AUDIO_ELEMENT_GET_INTERFACE (osxsink),
+      (void *) gst_osx_audio_sink_io_proc);
+
+  gst_osx_audio_sink_set_volume (osxsink);
+
+  ringbuffer->element = GST_OSX_AUDIO_ELEMENT_GET_INTERFACE (osxsink);
+  ringbuffer->device_id = osxsink->device_id;
+
+  return GST_RING_BUFFER (ringbuffer);
 }
 
-OSStatus
-gst_osx_audio_sink_io_proc (AudioDeviceID inDevice,
-    const AudioTimeStamp * inNow, const AudioBufferList * inInputData,
-    const AudioTimeStamp * inInputTime, AudioBufferList * outOutputData,
-    const AudioTimeStamp * inOutputTime, void *inClientData)
+/* HALOutput AudioUnit will request fairly arbitrarily-sized chunks of data,
+ * not of a fixed size. So, we keep track of where in the current ringbuffer
+ * segment we are, and only advance the segment once we've read the whole
+ * thing */
+static OSStatus
+gst_osx_audio_sink_io_proc (GstOsxRingBuffer * buf,
+    AudioUnitRenderActionFlags * ioActionFlags,
+    const AudioTimeStamp * inTimeStamp,
+    UInt32 inBusNumber, UInt32 inNumberFrames, AudioBufferList * bufferList)
 {
-  GstOsxRingBuffer *buf = GST_OSX_RING_BUFFER (inClientData);
-
   guint8 *readptr;
   gint readseg;
   gint len;
+  gint remaining = bufferList->mBuffers[0].mDataByteSize;
+  gint offset = 0;
 
-  if (gst_ring_buffer_prepare_read (GST_RING_BUFFER (buf), &readseg, &readptr,
-          &len)) {
-    outOutputData->mBuffers[0].mDataByteSize = len;
-    memcpy ((char *) outOutputData->mBuffers[0].mData, readptr, len);
+  while (remaining) {
+    if (!gst_ring_buffer_prepare_read (GST_RING_BUFFER (buf),
+            &readseg, &readptr, &len))
+      return 0;
 
-    /* clear written samples */
-    gst_ring_buffer_clear (GST_RING_BUFFER (buf), readseg);
+    len -= buf->segoffset;
 
-    /* we wrote one segment */
-    gst_ring_buffer_advance (GST_RING_BUFFER (buf), 1);
+    if (len > remaining)
+      len = remaining;
+
+    memcpy ((char *) bufferList->mBuffers[0].mData + offset,
+        readptr + buf->segoffset, len);
+
+    buf->segoffset += len;
+    offset += len;
+    remaining -= len;
+
+    if ((gint) buf->segoffset == GST_RING_BUFFER (buf)->spec.segsize) {
+      /* clear written samples */
+      gst_ring_buffer_clear (GST_RING_BUFFER (buf), readseg);
+
+      /* we wrote one segment */
+      gst_ring_buffer_advance (GST_RING_BUFFER (buf), 1);
+
+      buf->segoffset = 0;
+    }
   }
   return 0;
 }
@@ -318,34 +308,47 @@ gst_osx_audio_sink_osxelement_init (gpointer g_iface, gpointer iface_data)
 {
   GstOsxAudioElementInterface *iface = (GstOsxAudioElementInterface *) g_iface;
 
-  iface->io_proc = gst_osx_audio_sink_io_proc;
+  iface->io_proc = (AURenderCallback) gst_osx_audio_sink_io_proc;
 }
 
-/* entry point to initialize the plug-in
- * initialize the plug-in itself
- * register the element factories and pad templates
- * register the features
- *
- * exchange the string 'plugin' with your elemnt name
- */
-static gboolean
-plugin_init (GstPlugin * plugin)
+static void
+gst_osx_audio_sink_set_volume (GstOsxAudioSink * sink)
 {
-  gboolean ret;
+  if (!sink->audiounit)
+    return;
 
-  ret = gst_element_register (plugin, "osxaudiosink",
-      GST_RANK_PRIMARY, GST_TYPE_OSX_AUDIO_SINK);
-  return ret && gst_element_register (plugin, "osxaudiosrc",
-      GST_RANK_PRIMARY, GST_TYPE_OSX_AUDIO_SRC);
+  AudioUnitSetParameter (sink->audiounit, kHALOutputParam_Volume,
+      kAudioUnitScope_Global, 0, (float) sink->volume, 0);
 }
 
-/* this is the structure that gstreamer looks for to register plugins
- *
- * exchange the strings 'plugin' and 'Template plugin' with you plugin name and
- * description
- */
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "osxaudio",
-    "OSX Audio plugin",
-    plugin_init, VERSION, "LGPL", "GStreamer", "http://gstreamer.net/")
+static void
+gst_osx_audio_sink_select_device (GstOsxAudioSink * osxsink)
+{
+  OSStatus status;
+  UInt32 propertySize;
+
+  if (osxsink->device_id == kAudioDeviceUnknown) {
+    /* If no specific device has been selected by the user, then pick the
+     * default device */
+    GST_DEBUG_OBJECT (osxsink, "Selecting device for OSXAudioSink");
+    propertySize = sizeof (osxsink->device_id);
+    status =
+        AudioHardwareGetProperty (kAudioHardwarePropertyDefaultOutputDevice,
+        &propertySize, &osxsink->device_id);
+
+    if (status) {
+      GST_WARNING_OBJECT (osxsink,
+          "AudioHardwareGetProperty returned %d", (int) status);
+    } else {
+      GST_DEBUG_OBJECT (osxsink, "AudioHardwareGetProperty returned 0");
+    }
+
+    if (osxsink->device_id == kAudioDeviceUnknown) {
+      GST_WARNING_OBJECT (osxsink,
+          "AudioHardwareGetProperty: device_id is kAudioDeviceUnknown");
+    }
+
+    GST_DEBUG_OBJECT (osxsink, "AudioHardwareGetProperty: device_id is %lu",
+        (long) osxsink->device_id);
+  }
+}
